@@ -12,18 +12,20 @@
 #'@param fig_width Default width (in inches) for figures
 #'@param fig_height Default width (in inches) for figures
 #'@param fig_retina Scaling to perform for retina displays (defaults to 2, which
-#'  currently works for all widely used retina displays). Set to\code{NULL} to
-#'  prevent retina scaling. Note that this will always be\code{NULL} when
+#'  currently works for all widely used retina displays). Set to \code{NULL} to
+#'  prevent retina scaling. Note that this will always be \code{NULL} when
 #'  \code{keep_md} is specified (this is because \code{fig_retina} relies on
 #'  outputting HTML directly into the markdown document).
 #'@param fig_caption \code{TRUE} to render figures with captions
 #'@param dev Graphics device to use for figure output (defaults to png)
 #'@param code_folding Enable document readers to toggle the display of R code
-#'  chunks. Defaults to \code{"none"} which displays all code chunks (assuming
+#'  chunks. Specify \code{"none"} to display all code chunks (assuming
 #'  they were knit with \code{echo = TRUE}). Specify \code{"hide"} to hide all R
 #'  code chunks by default (users can show hidden code chunks either
 #'  individually or document-wide). Specify \code{"show"} to show all R code
 #'  chunks by default.
+#'@param code_download Embed the Rmd source code within the document and provide
+#'  a link that can be used by readers to download the code.
 #'@param smart Produce typographically correct output, converting straight
 #'  quotes to curly quotes, --- to em-dashes, -- to en-dashes, and ... to
 #'  ellipses.
@@ -118,11 +120,12 @@
 #'  control the behavior of the floating table of contents. Options include:
 #'
 #'  \itemize{ \item{\code{collapsed} (defaults to \code{TRUE}) controls whether
-#'  the table of contents appers with only the top-level (H2) headers. When
+#'  the table of contents appears with only the top-level (H2) headers. When
 #'  collapsed the table of contents is automatically expanded inline when
 #'  necessary.} \item{\code{smooth_scroll} (defaults to \code{TRUE}) controls
 #'  whether page scrolls are animated when table of contents items are navigated
-#'  to via mouse clicks.} }
+#'  to via mouse clicks.} \item{\code{print} (defaults to \code{TRUE}) controls
+#'  whether the table of contents appears when user prints out the HTML page.}}
 #'
 #'@section Tabbed Sections:
 #'
@@ -190,6 +193,7 @@ html_document <- function(toc = FALSE,
                           fig_caption = TRUE,
                           dev = 'png',
                           code_folding = c("none", "show", "hide"),
+                          code_download = FALSE,
                           smart = TRUE,
                           self_contained = TRUE,
                           theme = "default",
@@ -211,11 +215,11 @@ html_document <- function(toc = FALSE,
   # use section divs
   args <- c(args, "--section-divs")
 
-  # toc_float
-  if (!identical(toc_float, FALSE)) {
+  # table of contents
+  args <- c(args, pandoc_toc_args(toc, toc_depth))
 
-    # force toc on
-    toc <- TRUE
+  # toc_float
+  if (toc && !identical(toc_float, FALSE)) {
 
     # must have a theme
     if (is.null(theme))
@@ -223,7 +227,8 @@ html_document <- function(toc = FALSE,
 
     # resolve options
     toc_float_options <- list(collapsed = TRUE,
-                              smooth_scroll = TRUE)
+                              smooth_scroll = TRUE,
+                              print = TRUE)
     if (is.list(toc_float)) {
       toc_float_options <- merge_lists(toc_float_options, toc_float)
       toc_float <- TRUE
@@ -249,11 +254,10 @@ html_document <- function(toc = FALSE,
       args <- c(args, pandoc_variable_arg("toc_collapsed", "1"))
     if (toc_float_options$smooth_scroll)
       args <- c(args, pandoc_variable_arg("toc_smooth_scroll", "1"))
+    if (toc_float_options$print)
+      args <- c(args, pandoc_variable_arg("toc_print", "1"))
   }
 
-  # table of contents
-  args <- c(args, pandoc_toc_args(toc, toc_depth))
-
   # template path and assets
   if (identical(template, "default"))
     args <- c(args, "--template",
@@ -272,8 +276,18 @@ html_document <- function(toc = FALSE,
   # validate code_folding
   code_folding <- match.arg(code_folding)
 
-  # dummy pre_knit function so that merging of outputs works
-  pre_knit <- function(input, ...) {}
+  # capture the source code if requested
+  source_code <- NULL
+  source_file <- NULL
+  pre_knit <- function(input, ...) {
+    if (code_download) {
+      source_file <<- basename(input)
+      source_code <<- paste0(
+        '<div id="rmd-source-code">',
+        base64enc::base64encode(input),
+        '</div>')
+    }
+  }
 
   # pre-processor for arguments that may depend on the name of the
   # the input file AND which need to inject html dependencies
@@ -353,14 +367,33 @@ html_document <- function(toc = FALSE,
                                                   output_dir))
     }
 
+    # track whether we have a code menu
+    code_menu <- FALSE
+
     # code_folding
     if (code_folding %in% c("show", "hide")) {
       # must have a theme
       if (is.null(theme))
         stop("You must use a theme when specifying the 'code_folding' option")
       args <- c(args, pandoc_variable_arg("code_folding", code_folding))
+      code_menu <- TRUE
     }
 
+    # source_embed
+    if (code_download) {
+      if (is.null(theme))
+        stop("You must use a theme when specifying the 'code_download' option")
+      args <- c(args, pandoc_variable_arg("source_embed", source_file))
+      sourceCodeFile <- tempfile(fileext = ".html")
+      writeLines(source_code, sourceCodeFile)
+      args <- c(args, pandoc_include_args(after_body = sourceCodeFile))
+      code_menu <- TRUE
+    }
+
+    # code menu
+    if (code_menu)
+      args <- c(args, pandoc_variable_arg("code_menu", "1"))
+
     # content includes (we do this here so that user include-in-header content
     # goes after dependency generated content). make the paths absolute if
     # making a Shiny document so we can resolve them even if rendering
@@ -383,6 +416,7 @@ html_document <- function(toc = FALSE,
                             args = args),
     keep_md = keep_md,
     clean_supporting = self_contained,
+    pre_knit = pre_knit,
     post_knit = post_knit,
     pre_processor = pre_processor,
     base_format = html_document_base(smart = smart, theme = theme,
