@@ -55,6 +55,11 @@ html_document_base <- function(smart = TRUE,
 
   output_dir <- ""
 
+  # dummy pre_knit and post_knit functions so that merging of outputs works
+  pre_knit <- function(input, ...) {}
+  post_knit <- function(metadata, input_file, runtime, ...) {}
+
+  # pre_processor
   pre_processor <- function (metadata, input_file, runtime, knit_meta,
                              files_dir, output_dir) {
 
@@ -130,8 +135,16 @@ html_document_base <- function(smart = TRUE,
     output_str <- readLines(output_file, warn = FALSE, encoding = "UTF-8")
 
     # if we preserved chunks, restore them
-    if (length(preserved_chunks) > 0)
+    if (length(preserved_chunks) > 0) {
+      # Pandoc adds an empty <p></p> around the IDs of preserved chunks, and we
+      # need to remove these empty tags, otherwise we may have invalid HTML like
+      # <p><div>...</div></p>
+      for (i in names(preserved_chunks)) {
+        output_str <- gsub(paste0("<p>", i, "</p>"), i, output_str,
+                           fixed = TRUE, useBytes = TRUE)
+      }
       output_str <- restorePreserveChunks(output_str, preserved_chunks)
+    }
 
     if (copy_resources) {
       # The copy_resources flag copies all the resources referenced in the
@@ -163,6 +176,8 @@ html_document_base <- function(smart = TRUE,
     pandoc = pandoc_options(to = "html", from = NULL, args = args),
     keep_md = FALSE,
     clean_supporting = FALSE,
+    pre_knit = pre_knit,
+    post_knit = post_knit,
     pre_processor = pre_processor,
     intermediates_generator = intermediates_generator,
     post_processor = post_processor
