@@ -140,6 +140,25 @@ knitr_cache_dir <- function(file, pandoc_to) {
   paste(tools::file_path_sans_ext(file), "_cache/", pandoc_to, "/", sep = "")
 }
 
+get_knitr_hook_list <- function(hook_names = NULL) {
+  if (is.null(hook_names))
+    hook_names <- c("knit_hooks", "opts_chunk", "opts_hooks", "opts_knit")
+  knitr_ns <- asNamespace("knitr")
+  hook_list <- lapply(hook_names, function(hook_name) {
+    hooks <- get(hook_name, envir = knitr_ns, inherits = FALSE)
+    hooks$get()
+  })
+  names(hook_list) <- hook_names
+  hook_list
+}
+
+set_knitr_hook_list <- function(hook_list) {
+  knitr_ns <- asNamespace("knitr")
+  enumerate(hook_list, function(hook_name, hook_value) {
+    hook <- get(hook_name, envir = knitr_ns, inherits = FALSE)
+    hook$set(hook_value)
+  })
+}
 
 highlighters <- function() {
   c("default",
@@ -380,3 +399,164 @@ check_latexmk_version <- function(latexmk_path = find_program('latexmk')) {
     call. = FALSE
   )
 }
+
+n_bytes <- function(string) {
+  nchar(string, type = "bytes")
+}
+
+starts_with_bytes <- function(string, bytes) {
+  Encoding(string) <- Encoding(bytes) <- "bytes"
+  if (nchar(bytes) > nchar(string))
+    return(FALSE)
+  substring(string, 1, nchar(bytes)) == bytes
+}
+
+ends_with_bytes <- function(string, bytes) {
+  Encoding(string) <- Encoding(bytes) <- "bytes"
+  if (nchar(bytes) > nchar(string))
+    return(FALSE)
+  substring(string, nchar(string) - nchar(bytes) + 1, nchar(string)) == bytes
+}
+
+
+
+base64_encode_object <- function(object) {
+  object <- rapply(object, unclass, how = "list")
+  json <- charToRaw(jsonlite::toJSON(object, auto_unbox = TRUE))
+  base64enc::base64encode(json)
+}
+
+base64_decode_object <- function(encoded) {
+  json <- rawToChar(base64enc::base64decode(encoded))
+  jsonlite::fromJSON(json)
+}
+
+read_file <- function(path, binary = FALSE) {
+  n <- file.info(path)$size
+  if (binary) {
+    readBin(path, raw(), n)
+  } else {
+    readChar(path, n, TRUE)
+  }
+}
+
+surround <- function(string, with) {
+  paste(with, string, with, sep = "")
+}
+
+to_html_attributes <- function(data, on_empty = "", prefix = " ") {
+
+  if (inherits(data, "html"))
+    return(data)
+
+  if (!length(data))
+    return(on_empty)
+
+  # escape attribute contents
+  escaped <- unlist(lapply(data, function(el) {
+    htmltools::htmlEscape(join(as.character(el), collapse = " "), attribute = TRUE)
+  }))
+
+  # generate html attributes as string
+  quoted <- surround(escaped, with = "\"")
+  result <- join(names(data), quoted, sep = "=", collapse = " ")
+
+  # add prefix if necessary
+  if (nzchar(prefix))
+    result <- join(prefix, result)
+
+  # mark as html and return
+  class(result) <- "html"
+  result
+
+}
+
+to_css <- function(data, on_empty = "", prefix = "") {
+
+  if (inherits(data, "html"))
+    return(data)
+
+  if (!length(data))
+    return(on_empty)
+
+  # collapse vectors in data list
+  collapsed <- unlist(lapply(data, function(el) {
+    join(el, collapse = " ")
+  }))
+
+  # paste into single string
+  joined <- join(names(data), collapsed, sep = ": ", collapse = "; ")
+
+  # add prefix
+  if (nzchar(prefix))
+    joined <- join(prefix, joined)
+
+  # return with trailing semi-colon
+  result <- join(joined, ";", sep = "")
+  class(result) <- "html"
+  result
+}
+
+rbind_list <- function(data) {
+  result <- do.call(mapply, c(c, data, USE.NAMES = FALSE, SIMPLIFY = FALSE))
+  names(result) <- names(data[[1]])
+  as.data.frame(result, stringsAsFactors = FALSE)
+}
+
+enumerate <- function(data, f, ...) {
+  lapply(seq_along(data), function(i) {
+    f(names(data)[[i]], data[[i]], ...)
+  })
+}
+
+insert <- function(vector, index, ...) {
+
+  dots <- list(...)
+  mode(dots) <- mode(vector)
+  n <- length(vector)
+
+  result <- if (index == 0) {
+    c(dots, vector)
+  } else if (index == n) {
+    c(vector, dots)
+  } else {
+    c(vector[1:index], dots, vector[(index + 1):n])
+  }
+
+  result
+}
+
+replace_binding <- function(binding, package, override) {
+  # override in namespace
+  if (!requireNamespace(package, quietly = TRUE))
+    stop(sprintf("Failed to load namespace for package '%s'", package))
+
+  namespace <- asNamespace(package)
+
+  # get reference to original binding
+  original <- get(binding, envir = namespace)
+
+  # replace the binding
+  if (is.function(override))
+    environment(override) <- namespace
+
+  do.call("unlockBinding", list(binding, namespace))
+  assign(binding, override, envir = namespace)
+  do.call("lockBinding", list(binding, namespace))
+
+  # if package is attached, override there as well
+  search_name <- paste("package", package, sep = ":")
+  if (search_name %in% search()) {
+    env <- as.environment(search_name)
+    do.call("unlockBinding", list(binding, env))
+    assign(binding, override, envir = env)
+    do.call("lockBinding", list(binding, env))
+  }
+
+  # return original
+  original
+}
+
+join <- function(..., sep = "", collapse = "") {
+  paste(..., sep = sep, collapse = collapse)
+}
